from dataclasses import dataclass
from pathlib import Path

import moviepy.editor as mp


@dataclass
class Video:
    path: Path
    clip: mp.VideoFileClip

    @classmethod
    def from_path(cls, path: Path) -> "Video":
        return cls(path=path, clip=mp.VideoFileClip(str(path.resolve())))

    def transform_clip(self, begin_after: float, scale: float) -> mp.VideoFileClip:
        return self.clip.subclip(begin_after, self.clip.duration).resize(scale)



from pathlib import Path

from terminhtml_recorder.formats import OutputFormat
from terminhtml_recorder.temp_path import create_temp_path
from tests.checks.gif import assert_is_valid_gif
from tests.fixtures.temp_folder import temp_folder
from tests.generate import create_terminhtml_demo_gif


def test_recorder(temp_folder: Path):
    gif = create_terminhtml_demo_gif(temp_folder / "terminhtml-demo.gif")
    # TODO: Verify GIF output in tests
    assert gif.format == OutputFormat.GIF
    assert_is_valid_gif(gif.path)
    assert gif.path.parent == temp_folder



import shutil
from dataclasses import dataclass
from pathlib import Path
from typing import Callable, Union

from playwright.sync_api import Locator, Page, sync_playwright

from terminhtml_recorder.converter import convert_video_to_gif
from terminhtml_recorder.formats import OutputFormat
from terminhtml_recorder.logger import log
from terminhtml_recorder.temp_path import create_temp_path
from terminhtml_recorder.video import Video


@dataclass
class Recording:
    video: Video
    format: OutputFormat

    @classmethod
    def from_path(cls, path: Path, format: OutputFormat) -> "Recording":
        return cls(video=Video.from_path(path), format=format)

    @property
    def path(self) -> Path:
        return self.video.path

    def convert_to(
        self,
        new_format: OutputFormat,
        out_path: Path,
        begin_after: float = 1.1,
        resize: float = 0.7,
        fps: int = 10,
    ) -> "Recording":
        clip = self.video.transform_clip(begin_after, resize)
        if new_format == self.format:
            shutil.copy(self.path, out_path)
            # TODO: need to trim for delay in webm output
            return Recording.from_path(path=out_path, format=new_format)
        elif new_format == OutputFormat.GIF:
            convert_video_to_gif(clip, out_path, fps=fps)
            return Recording.from_path(path=out_path, format=new_format)
        raise NotImplementedError(
            f"Conversion from {self.format} to {new_format} not implemented"
        )


@dataclass
class PageLocators:
    speed_up: Locator
    speed_down: Locator
    restart: Locator

    @classmethod
    def from_page(cls, page: Page) -> "PageLocators":
        return cls(
            speed_up=page.locator("text=►"),
            speed_down=page.locator("text=◄"),
            restart=page.locator("text=restart ↻"),
        )


PageInteractor = Callable[[PageLocators], None]


def default_page_interactor(page_locators: PageLocators) -> None:
    page_locators.restart.wait_for()


class TerminHTMLRecorder:
    def __init__(self, html: str, interactor: PageInteractor = default_page_interactor):
        self.html = html
        self.interactor = interactor

    def record(
        self,
        out_path: Union[str, Path],
        format: OutputFormat = OutputFormat.GIF,
        begin_after: float = 1.1,
        resize: float = 0.7,
        fps: int = 10,
    ) -> Recording:
        with create_temp_path() as temp_path:
            log.info(f"Creating recording in {temp_path}")
            html_path = temp_path / "termin.html"
            html_path.write_text(self.html)

            with sync_playwright() as p:
                browser = p.chromium.launch()
                dimensions = dict(width=800, height=530)
                context = browser.new_context(
                    viewport=dimensions,
                    record_video_dir=str(temp_path.resolve()),
                    record_video_size=dimensions,
                )
                page = context.new_page()
                page.goto(html_path.as_uri())
                locators = PageLocators.from_page(page)
                self.interactor(locators)
                context.close()
                browser.close()

            for video in temp_path.glob("*.webm"):
                return Recording.from_path(
                    path=video,
                    format=OutputFormat.WEBM,
                ).convert_to(
                    format,
                    Path(out_path),
                    begin_after=begin_after,
                    resize=resize,
                    fps=fps,
                )

            raise ValueError(f"No video found in temp_path {temp_path}")



from pathlib import Path

TESTS_DIR = Path(__file__).parent
INPUT_FILES_DIR = TESTS_DIR / "input_files"

TERMINHTML_DEMO_GIF = INPUT_FILES_DIR / "terminhtml-demo.gif"
TERMINHTML_DEMO_HTML = INPUT_FILES_DIR / "terminhtml-demo.html"



from pathlib import Path


def assert_is_valid_gif(path: Path):
    """
    Check if the given path is a valid GIF.
    """
    with path.open("rb") as f:
        assert f.read(3) == b"GIF"
    assert path.exists()
    assert path.is_file()
    assert path.stat().st_size > 0



"""
Create video and image files via TerminHTML rather than HTML/CSS/JS
"""
from terminhtml_recorder.formats import OutputFormat
from terminhtml_recorder.recorder import (
    PageLocators,
    TerminHTMLRecorder,
    default_page_interactor,
)



import shlex
from pathlib import Path
from typing import Sequence

from click.testing import Result
from cliconf import CLIConf
from cliconf.testing import CLIRunner

from terminhtml_recorder.cli import cli
from tests import ext_click
from tests.checks.gif import assert_is_valid_gif
from tests.config import TERMINHTML_DEMO_HTML
from tests.fixtures.temp_folder import temp_folder

runner = CLIRunner()


class CLIRunnerException(Exception):
    pass


def run(args: Sequence[str]) -> Result:
    result = runner.invoke(cli, args)
    if result.exit_code != 0:
        output = ext_click.result_to_message(result)
        command = shlex.join([cli.info.name, *args])
        raise CLIRunnerException(
            f"{command} with exited with code {result.exit_code}.\n{output}"
        )
    return result


def test_record_from_file(temp_folder: Path):
    out_path = temp_folder / "output.gif"
    run(["-i", str(TERMINHTML_DEMO_HTML), "-o", str(out_path)])
    assert_is_valid_gif(out_path)



"""
Run this script to generate the files in input_files so that
they can be manually QA'ed.
"""

from pathlib import Path

from terminhtml.main import TerminHTML

from terminhtml_recorder.formats import OutputFormat
from terminhtml_recorder.recorder import Recording, TerminHTMLRecorder
from tests.config import TERMINHTML_DEMO_GIF, TERMINHTML_DEMO_HTML


def create_terminhtml_demo_gif(out_path: Path = TERMINHTML_DEMO_GIF) -> Recording:
    term = TerminHTML.from_commands(
        ["python -m terminhtml.demo_output"],
        prompt_matchers=["\\[0m: "],
        input=["Nick DeRobertis"],
    )
    text = term.to_html()
    TERMINHTML_DEMO_HTML.write_text(text)
    recorder = TerminHTMLRecorder(text)
    recording = recorder.record(out_path, OutputFormat.GIF)
    return recording


if __name__ == "__main__":
    create_terminhtml_demo_gif()



import os
import sys
from pathlib import Path
from typing import Optional

import typer
from cliconf import CLIAppConfig, CLIConf, configure
from pyappconf import ConfigFormats

from terminhtml_recorder.exc import NoHTMLFoundException
from terminhtml_recorder.formats import OutputFormat
from terminhtml_recorder.recorder import (
    PageInteractor,
    TerminHTMLRecorder,
    default_page_interactor,
)

cli = CLIConf(
    name="terminrec",
    help="Record TerminHTML animated output to a video file.",
)


@cli.command()
@configure(
    CLIAppConfig(
        app_name="terminhtml-recorder",
        config_name="terminrec",
        default_format=ConfigFormats.PY,
        py_config_imports=[
            "from pathlib import Path",
            "from terminhtml_recorder import PageLocators, default_page_interactor, OutputFormat",
            "import terminhtml_recorder.recorder",
            "from terminhtml_recorder.cli import record",
        ],
    )
)
def record(
    in_path: Optional[Path] = typer.Option(
        None,
        "--input-path",
        "-i",
        help="Path of file containing TerminHTML output. If not passed, must be piped to stdin.",
    ),
    out_path: Optional[Path] = typer.Option(
        None,
        "--output-path",
        "-o",
        help="Path of file to write recording to. Defaults to terminhtml.<output-format>",
        show_default=False,
    ),
    output_format: OutputFormat = typer.Option(
        OutputFormat.GIF, "--format", "-f", help="Format of recording"
    ),
    begin_after: float = typer.Option(
        1.1,
        "--begin-after",
        "-b",
        help="Delay from browser start until recording start. Increase if you see an uninitialized terminal at the start. Decrease if you don't see the beginning of typing.",
    ),
    resize: float = typer.Option(
        0.7, "--resize", "-r", help="Resize video to this fraction of original size"
    ),
    fps: int = typer.Option(10, "--fps", "-p", help="Frames per second of video"),
    interactor: PageInteractor = default_page_interactor,
):
    """
    Record TerminHTML animated output to a video file.
    """
    if in_path is not None:
        html = in_path.read_text()
    # Else if input was piped to stdin
    elif not sys.stdin.isatty() and not os.getenv("PYCHARM_HOSTED"):
        # Read HTML from stdin
        html = sys.stdin.read()
    else:
        raise NoHTMLFoundException(
            f"No HTML passed. Must either specify an input file or pipe HTML to stdin."
        )

    if not html:
        raise NoHTMLFoundException(
            f"Empty HTML found. Must either specify a valid input file or pipe HTML to stdin."
        )

    out_path = out_path or Path("terminhtml").with_suffix(f".{output_format.value}")

    recorder = TerminHTMLRecorder(html, interactor)
    recorder.record(
        out_path, format=output_format, begin_after=begin_after, resize=resize, fps=fps
    )


if __name__ == "__main__":
    cli()



from pathlib import Path

import pytest

from terminhtml_recorder.temp_path import create_temp_path


@pytest.fixture
def temp_folder() -> Path:
    with create_temp_path() as temp_path:
        yield temp_path



from pathlib import Path

import moviepy.editor as mp

from terminhtml_recorder.logger import log


def convert_video_to_gif(clip: mp.VideoFileClip, out_path: Path, fps: int = 10) -> None:
    clip.write_gif(str(out_path.resolve()), program="ffmpeg", fps=fps)
    log.info(f"Demo output gif saved to {out_path}")
